@@ -2,13 +2,13 @@ package Blosxom::Header;
 use 5.008_001;
 use strict;
 use warnings;
+use Carp;
 use Exporter 'import';
-use List::Util qw(first);
 
-our $VERSION     = '0.01016';
-our @EXPORT_OK   = qw( get_header set_header delete_header exists_header );
-our %EXPORT_TAGS = ( all => [ @EXPORT_OK ] );
+our $VERSION   = '0.02003';
+our @EXPORT_OK = qw( get_header set_header push_cookie delete_header exists_header );
 
+# the alias of Blosxom::Header::Object->new
 sub new {
     require Blosxom::Header::Object;
     Blosxom::Header::Object->new( $_[1] );
@@ -16,61 +16,84 @@ sub new {
 
 sub get_header {
     my $header_ref = shift;
-    my $key        = _lc( shift );
+    my $key        = _norm( shift );
 
-    # if any key matches $key, returns the value
+    my @values;
     while ( my ( $k, $v ) = each %{ $header_ref } ) {
-        return $v if _lc( $k ) eq $key;
+        push @values, $v if $key eq _norm( $k );
     }
 
+    return unless @values;
+    carp "Multiple elements specify the $key header." if @values > 1;
+
+    my $value = shift @values;
+    return $value unless ref $value eq 'ARRAY';
+    wantarray ? @{ $value } : $value->[0];
+}
+
+sub delete_header {
+    my $header_ref = shift;
+    my $key        = _norm( shift );
+
+    # deletes elements whose key matches $key
+    my @keys = grep { _norm( $_ ) eq $key } keys %{ $header_ref };
+    delete @{ $header_ref }{ @keys };
+
     return;
 }
 
 sub set_header {
-    my ( $header_ref, $key, $value ) = @_;
+    my $header_ref = shift;
+    my $key        = _norm( shift );
+    my $value      = shift;
 
-    # if any key matches $key, replaces the value with $value
-    my @keys      = keys %{ $header_ref };
-    my $first_key = first { _lc( $_ ) eq _lc( $key ) } @keys;
-    $header_ref->{ $first_key || $key } = $value;
+    delete_header( $header_ref, $key );
+    $header_ref->{ $key } = $value;
 
     return;
 }
 
 sub exists_header {
     my $header_ref = shift;
-    my $key        = _lc( shift );
+    my $key        = _norm( shift );
 
-    # if any key matches $key, returns true
+    my $exists = 0;
     for my $k ( keys %{ $header_ref } ) {
-        return 1 if _lc( $k ) eq $key;
+        $exists++ if _norm( $k ) eq $key;
     }
 
-    return 0;
+    carp "$exists elements specify the $key field." if $exists > 1;
+    $exists;
 }
 
-sub delete_header {
-    my $header_ref = shift;
-    my $key        = _lc( shift );
-
-    # deletes an element whose key matches $key
-    my @keys = grep { _lc( $_ ) eq $key } keys %{ $header_ref };
-    delete @{ $header_ref }{ @keys };
-
+sub push_cookie {
+    my ( $header_ref, $cookie ) = @_;
+    my @cookies = get_header( $header_ref, 'cookie' ) || ();
+    push @cookies, $cookie;
+    set_header( $header_ref, 'cookie' => \@cookies );
     return;
 }
 
-# returns a lowercased version of a given string
-sub _lc {
-    my $key = lc shift;
+{
+    # suppose read-only
+    my %alias_of = (
+        'content-type' => 'type',
+        'set-cookie'   => 'cookie',
+    );
 
-    # get rid of an initial dash if exists
-    $key =~ s{^\-}{};
+    # normalize a given key
+    sub _norm {
+        my $key = lc shift;
 
-    # use dashes instead of underscores
-    $key =~ tr{_}{-};
+        # get rid of an initial dash if exists
+        $key =~ s{^\-}{};
 
-    return $key;
+        # use dashes instead of underscores
+        $key =~ tr{_}{-};
+
+        # returns the alias of $key if exists
+        $alias_of{ $key } || $key;
+    }
 }
 
 1;
@@ -83,20 +106,38 @@ Blosxom::Header - Missing interface to modify HTTP headers
 
 =head1 SYNOPSIS
 
-  use Blosxom::Header qw(get_header set_header exists_header delete_header);
+  use Blosxom::Header qw(
+      get_header
+      set_header
+      delete_header
+      exists_header
+      push_cookie
+  );
 
   # Procedural interface
+
   my $value = get_header( $blosxom::header, 'foo' );
   my $bool  = exists_header( $blosxom::header, 'foo' );
-  set_header( $blosxom::header, 'bar' => 'baz' );
+
+  set_header( $blosxom::header, foo => 'bar' );
   delete_header( $blosxom::header, 'foo' );
 
+  my @cookies = get_header( $blosxom::header, 'Set-Cookie' );
+  push_cookie( $blosxom::header, 'foo' );
+
   # Object-oriented interface
+
   my $h     = Blosxom::Header->new( $blosxom::header );
-  my $value = $h->get('foo');
-  my $bool  = $h->exists('foo');
-  $h->set( bar => 'baz' );
-  $h->delete('foo');
+  my $value = $h->get( 'foo' );
+  my $bool  = $h->exists( 'foo' );
+
+  $h->set( foo => 'bar' );
+  $h->delete( 'foo' );
+
+  my @cookies = $h->get( 'Set-Cookie' );
+  $h->push_cookie( 'foo' );
+
+  $h->header; # same reference as $blosxom::header
 
 =head1 DESCRIPTION
 
@@ -111,11 +152,14 @@ When plugin developers modify HTTP headers, they must write as follows:
   $blosxom::header->{'-status'} = '304 Not Modified';
 
 It's obviously bad practice.
-Blosxom misses the interface to modify them.
+Multiple elements may specify the same field:
 
-This module provieds you a procedural interface or an object-oriented one.
-You don't need to mind whether to put a dash before a key,
-nor whether to make a key lowercased or L<camelized|String::CamelCase>.
+  $blosxom::header->{'-status'} = '304 Not Modified';
+  $blosxom::header->{'status' } = '304 Not Modified';
+  $blosxom::header->{'-Status'} = '304 Not Modified';
+  $blosxom::header->{'Status' } = '304 Not Modified';
+
+Blosxom misses the interface to modify HTTP headers.
 
 =head2 SUBROUTINES
 
@@ -123,21 +167,29 @@ The following are exported on demand.
 
 =over 4
 
-=item get_header( $blosxom::header, 'foo' )
+=item $value = get_header( $blosxom::header, 'foo' )
 
 Returns a value of the specified HTTP header.
 
+=item @cookies = get_header( $blosxom::header, 'Set-Cookie' )
+
+Returns values of the Set-Cookie headers.
+
 =item set_header( $blosxom::header, 'foo' => 'bar' )
 
 Sets a value of the specified HTTP header.
 
-=item exists_header( $blosxom::header, 'foo' )
+=item $bool = exists_header( $blosxom::header, 'foo' )
 
 Returns a Boolean value telling whether the specified HTTP header exists.
 
 =item delete_header( $blosxom::header, 'foo' )
 
-Deletes the specified element from HTTP headers.
+Deletes all the specified elements from HTTP headers.
+
+=item push_cookie( $blosxom::header, 'foo' )
+
+Pushes the Set-Cookie header onto HTTP headers.
 
 =back
 
@@ -148,12 +200,15 @@ Deletes the specified element from HTTP headers.
 =item $h = Blosxom::Header->new( $blosxom::header )
 
 Creates a new Blosxom::Header object.
+Must pass a reference to hash.
 
-=item $h->exists( 'foo' )
+=item $bool = $h->exists( 'foo' )
 
 A synonym for exists_header.
 
-=item $h->get( 'foo' )
+=item $value = $h->get( 'foo' )
+
+=item @cookies = $h->get( 'Set-Cookie' )
 
 A synonym for get_header.
 
@@ -165,6 +220,14 @@ A synonym for delete_header.
 
 A synonym for set_header.
 
+=item $h->push_cookie( 'foo' )
+
+A synonym for push_cookie.
+
+=item $h->header
+
+Returns the same reference as $blosxom::header.
+
 =back
 
 =head1 EXAMPLES
@@ -193,7 +256,7 @@ If not provided, defaults to ISO-8859-1.
 
 =item cookie
 
-Represents the Set-Cookie header.
+Represents the Set-Cookie headers.
 The parameter can be an arrayref or a string.
 
   $h->set( cookie => [$cookie1, $cookie2] );
@@ -236,7 +299,7 @@ The parameter can be an arrayref or a space-delimited string.
 
 In either case, the outgoing header will be formatted as:
 
-  P3P: policyref="/w3c/p3p.xml" cp="CAO DSP LAW CURa"
+  P3P: policyref="/w3c/p3p.xml" CP="CAO DSP LAW CURa"
 
 =item type
 
